@@ -1,14 +1,9 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
-import codecs
 import hashlib
-import json
 import json.decoder
 import logging
 import ntpath
+import random
 
-import six
 from docker.errors import DockerException
 from docker.utils import parse_bytes as sdk_parse_bytes
 
@@ -21,12 +16,6 @@ json_decoder = json.JSONDecoder()
 log = logging.getLogger(__name__)
 
 
-def get_output_stream(stream):
-    if six.PY3:
-        return stream
-    return codecs.getwriter('utf-8')(stream)
-
-
 def stream_as_text(stream):
     """Given a stream of bytes or text, if any of the items in the stream
     are bytes convert them to text.
@@ -35,13 +24,13 @@ def stream_as_text(stream):
     of byte streams.
     """
     for data in stream:
-        if not isinstance(data, six.text_type):
+        if not isinstance(data, str):
             data = data.decode('utf-8', 'replace')
         yield data
 
 
-def line_splitter(buffer, separator=u'\n'):
-    index = buffer.find(six.text_type(separator))
+def line_splitter(buffer, separator='\n'):
+    index = buffer.find(str(separator))
     if index == -1:
         return None
     return buffer[:index + 1], buffer[index + 1:]
@@ -56,7 +45,7 @@ def split_buffer(stream, splitter=None, decoder=lambda a: a):
     of the input.
     """
     splitter = splitter or line_splitter
-    buffered = six.text_type('')
+    buffered = ''
 
     for data in stream_as_text(stream):
         buffered += data
@@ -101,7 +90,7 @@ def json_stream(stream):
 
 
 def json_hash(obj):
-    dump = json.dumps(obj, sort_keys=True, separators=(',', ':'))
+    dump = json.dumps(obj, sort_keys=True, separators=(',', ':'), default=lambda x: x.repr())
     h = hashlib.sha256()
     h.update(dump.encode('utf8'))
     return h.hexdigest()
@@ -127,7 +116,7 @@ def parse_nanoseconds_int(value):
 
 
 def build_string_dict(source_dict):
-    return dict((k, str(v if v is not None else '')) for k, v in source_dict.items())
+    return {k: str(v if v is not None else '') for k, v in source_dict.items()}
 
 
 def splitdrive(path):
@@ -143,3 +132,45 @@ def parse_bytes(n):
         return sdk_parse_bytes(n)
     except DockerException:
         return None
+
+
+def unquote_path(s):
+    if not s:
+        return s
+    if s[0] == '"' and s[-1] == '"':
+        return s[1:-1]
+    return s
+
+
+def generate_random_id():
+    while True:
+        val = hex(random.getrandbits(32 * 8))[2:-1]
+        try:
+            int(truncate_id(val))
+            continue
+        except ValueError:
+            return val
+
+
+def truncate_id(value):
+    if ':' in value:
+        value = value[value.index(':') + 1:]
+    if len(value) > 12:
+        return value[:12]
+    return value
+
+
+def unique_everseen(iterable, key=lambda x: x):
+    "List unique elements, preserving order. Remember all elements ever seen."
+    seen = set()
+    for element in iterable:
+        unique_key = key(element)
+        if unique_key not in seen:
+            seen.add(unique_key)
+            yield element
+
+
+def truncate_string(s, max_chars=35):
+    if len(s) > max_chars:
+        return s[:max_chars - 2] + '...'
+    return s
