@@ -1,6 +1,3 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import logging
 
 import docker
@@ -12,7 +9,7 @@ from compose.cli.formatter import ConsoleWarningFormatter
 from compose.cli.main import build_one_off_container_options
 from compose.cli.main import call_docker
 from compose.cli.main import convergence_strategy_from_opts
-from compose.cli.main import filter_containers_to_service_names
+from compose.cli.main import filter_attached_containers
 from compose.cli.main import get_docker_start_call
 from compose.cli.main import setup_console_handler
 from compose.cli.main import warn_for_swarm_mode
@@ -25,7 +22,7 @@ def mock_container(service, number):
         container.Container,
         service=service,
         number=number,
-        name_without_project='{0}_{1}'.format(service, number))
+        name_without_project='{}_{}'.format(service, number))
 
 
 @pytest.fixture
@@ -35,9 +32,9 @@ def logging_handler():
     return logging.StreamHandler(stream=stream)
 
 
-class TestCLIMainTestCase(object):
+class TestCLIMainTestCase:
 
-    def test_filter_containers_to_service_names(self):
+    def test_filter_attached_containers(self):
         containers = [
             mock_container('web', 1),
             mock_container('web', 2),
@@ -46,17 +43,29 @@ class TestCLIMainTestCase(object):
             mock_container('another', 1),
         ]
         service_names = ['web', 'db']
-        actual = filter_containers_to_service_names(containers, service_names)
+        actual = filter_attached_containers(containers, service_names)
         assert actual == containers[:3]
 
-    def test_filter_containers_to_service_names_all(self):
+    def test_filter_attached_containers_with_dependencies(self):
+        containers = [
+            mock_container('web', 1),
+            mock_container('web', 2),
+            mock_container('db', 1),
+            mock_container('other', 1),
+            mock_container('another', 1),
+        ]
+        service_names = ['web', 'db']
+        actual = filter_attached_containers(containers, service_names, attach_dependencies=True)
+        assert actual == containers
+
+    def test_filter_attached_containers_all(self):
         containers = [
             mock_container('web', 1),
             mock_container('db', 1),
             mock_container('other', 1),
         ]
         service_names = []
-        actual = filter_containers_to_service_names(containers, service_names)
+        actual = filter_attached_containers(containers, service_names)
         assert actual == containers
 
     def test_warning_in_swarm_mode(self):
@@ -126,7 +135,7 @@ class TestCLIMainTestCase(object):
         assert expected_docker_start_call == docker_start_call
 
 
-class TestSetupConsoleHandlerTestCase(object):
+class TestSetupConsoleHandlerTestCase:
 
     def test_with_tty_verbose(self, logging_handler):
         setup_console_handler(logging_handler, True)
@@ -146,7 +155,7 @@ class TestSetupConsoleHandlerTestCase(object):
         assert type(logging_handler.formatter) == logging.Formatter
 
 
-class TestConvergeStrategyFromOptsTestCase(object):
+class TestConvergeStrategyFromOptsTestCase:
 
     def test_invalid_opts(self):
         options = {'--force-recreate': True, '--no-recreate': True}
@@ -180,7 +189,7 @@ def mock_find_executable(exe):
 
 
 @mock.patch('compose.cli.main.find_executable', mock_find_executable)
-class TestCallDocker(object):
+class TestCallDocker:
     def test_simple_no_options(self):
         with mock.patch('subprocess.call') as fake_call:
             call_docker(['ps'], {}, {})
